// `define use_behavioral_verilog

module alu
	(input [7:0] a,
	 input [7:0] b,
	 input cin,
	 input addSub,
	 output [7:0] result,
	 output cout);

`ifdef use_behavioral_verilog
	
	wire [7:0] bnotb = addSub ? b : ~b;
	
	assign { cout, result } = a + bnotb + cin; // better synthesis result than cin + a + bnotb
	
`else
	
	lpm_add_sub	lpm_add_sub_component (
				.dataa (a),
				.add_sub (addSub),
				.datab (b),
				.cin (cin),
				.cout (cout),
				.result (result)
				// synopsys translate_off
				,
				.aclr (),
				.clken (),
				.clock (),
				.overflow ()
				// synopsys translate_on
				);
	defparam
		lpm_add_sub_component.lpm_direction = "UNUSED",
		lpm_add_sub_component.lpm_hint = "ONE_INPUT_IS_CONSTANT=NO,CIN_USED=YES",
		lpm_add_sub_component.lpm_representation = "UNSIGNED",
		lpm_add_sub_component.lpm_type = "LPM_ADD_SUB",
		lpm_add_sub_component.lpm_width = 8;
  
`endif	

endmodule



