* = $0000

; SEF macros for setting/clearing ALU condition codes
!macro sez { sef #$F8 }
!macro clz { sef #$70 }
!macro sec { sef #$F4 }
!macro clc { sef #$B0 }
!macro sev { sef #$F2 }
!macro clv { sef #$D0 }
!macro sen { sef #$F1 }
!macro cln { sef #$E0 }

; Constants
SUCCESS = $D3

; ===== JUMP
	lda #0
	jmp jmp_ahead
	halt

jmp_back:
	jmp jmp_done
		
jmp_ahead:
	jmp jmp_back
	halt
	
jmp_done:		

; ===== CONDITIONAL BRANCHES	
	+sez
	bne +
	beq ++
+	halt
++  +clz
	beq +
	bne ++
+	halt
++	

	+sec
	bcc +
	bcs ++
+	halt
++  +clc
	bcs +
	bcc ++
+	halt
++	

	+sev
	bvc +
	bvs ++
+	halt
++  +clv
	bvs +
	bvc ++
+	halt
++	

	+sen
	bpl +
	bmi ++
+	halt
++  +cln
	bmi +
	bpl ++
+	halt
++	

; ===== LDA, CMP
	lda #$00 ; zero, plus
	bne +
	bmi +
	jmp ++
+	halt
++	lda #$7F ; non-zero, plus
	beq +
	bmi +
	jmp ++
+	halt
++	lda #$B8 ; non-zero, minus
	beq +
	bpl +
	jmp ++
+	halt
++	cmp #$B8 ; equal
	bne +
	bmi +
	bcc +
	jmp ++
+	halt
++	cmp #$07 ; greater than
	beq +
	bpl +
	bcc +
	jmp ++
+	halt
++	cmp #$FF ; less than
	beq +
	bpl +
	bcs +
	jmp ++
+	halt

; ===== STA
++	sta $0300
	lda #$4E
	sta $0301
	cmp #$4E
	beq +
	halt
+	lda $0300
	cmp #$B8
	beq +
	halt
	lda $0301
	cmp #$4E
	beq +
	halt
	
; ===== CMP abs
+	lda #$4E
	cmp $0301
	beq +
	halt
+	lda #$B8
	cmp $0300
	beq +
	halt
	
; ===== AND
+	lda #$E3
	and #$39 ; result should be $21
	beq +
	bmi +
	jmp ++
+	halt
++	cmp #$21
	beq +
	halt

; ===== AND abs
+	lda #$6F
	and $0300 ; result should be $28
	beq +
	bmi +
	jmp ++
+	halt
++	cmp #$28
	beq +
	halt
+
	
; ===== ORA
+	lda #$E3
	ora #$39 ; result should be $FB
	beq +
	bpl +
	jmp ++
+	halt
++	cmp #$FB
	beq +
	halt

; ===== ORA abs
+	lda #$6F
	ora $0300 ; result should be $FF
	beq +
	bpl +
	jmp ++
+	halt
++	cmp #$FF
	beq +
	halt
+

; ===== EOR
+	lda #$E3
	eor #$39 ; result should be $DA
	beq +
	bpl +
	jmp ++
+	halt
++	cmp #$DA
	beq +
	halt

; ===== EOR abs
+	lda #$6F
	eor $0300 ; result should be $D7
	beq +
	bpl +
	jmp ++
+	halt
++	cmp #$D7
	beq +
	halt

; ===== ADC
+	lda #$33
	+clc
	adc #$41 ; basic add
	bvs +
	beq +
	bcs +
	bmi +
	jmp ++
+	halt
++	cmp #$74
	beq +
	halt
+	lda #$33
	+sec
	adc #$41 ; add with carry
	bvs +
	beq +
	bcs +
	bmi +
	jmp ++
+	halt
++	cmp #$75
	beq +
	halt
+	lda #$33
	+clc
	adc #$50 ; should set N,V
	bvc +
	beq +
	bcs +
	bpl +
	jmp ++
+	halt	
++  cmp #$83
	beq +
	halt
+	lda #$FF
	+clc
	adc #$FE ; should set C,N
	bvs +
	beq +
	bcc +
	bpl +
	jmp ++
+	halt	
++  cmp #$FD
	beq +
	halt
+	lda #$70
	+clc
	adc #$90 ; should set Z,C
	bvs +
	bne +
	bcc +
	bmi +
	jmp ++
+	halt	
++  cmp #$00	
	beq +
	halt
	
; ===== ADC abs
+	lda #$34
	+clc
	adc $0300 ; result should be $EC
	cmp #$EC
	beq +
	halt
	
; ===== SBC
+	lda #$80
	+sec
	sbc #$13 ; result should be $6D, with C,V set
	beq +
	bmi +
	bcc +
	bvc +
	jmp ++
+	halt
++	cmp #$6D
	beq +
	halt
+	lda #$80
	+sec
	sbc #$80 ; result should be 0, with Z,C set
	bne +
	bmi +
	bcc +
	bvs +
	jmp ++
+	halt
++	lda #$80
	+sec
	sbc #$81 ; result should be $FF, with N set
	beq +
	bpl +
	bcs +
	bvs +
	jmp ++
+	halt
++	cmp #$FF
	beq +
	halt
+	lda #$7F
	+clc
	sbc #$7E ; result should be 0, with Z,C set
	bne +
	bmi +
	bcc +
	bvs +
	jmp ++
+	halt
++	lda #$7F
	+clc
	sbc #$7F ; result should be $FF, with N set
	beq +
	bpl +
	bcs +
	bvs +
	jmp ++
+	halt
++	lda #$13
	+sec
	sbc #$81 ; result should be $92, with N,V set
	beq +
	bpl +
	bcs +
	bvc +
	jmp ++
+	halt
++	cmp #$92
	beq +
	halt
	
; ===== SBC abs
+	lda #$34
	+sec
	sbc $0300 ; result should be $7C
	beq +
	bmi +
	bcs +
	bvs +
	jmp ++
+	halt
++	cmp #$7C
	beq +
	halt
+	

done:
	lda #SUCCESS
	halt
	